@@ -11,6 +11,7 @@ Serialization utilities.
 from __future__ import absolute_import
 
 import codecs
+import os
 import sys
 
 import pickle as pypickle
@@ -64,6 +65,14 @@ else:
     except ImportError:
         from StringIO import StringIO  as BytesIO # noqa
 
+#: Kombu requires Python 2.5 or later so we use protocol 2 by default.
+#: There's a new protocol (3) but this is only supported by Python 3.
+pickle_protocol = int(os.environ.get('PICKLE_PROTOCOL', 2))
+
+#: Kombu requires Python 2.5 or later so we use protocol 2 by default.
+#: There's a new protocol (3) but this is only supported by Python 3.
+pickle_protocol = int(os.environ.get('PICKLE_PROTOCOL', 2))
+
 
 def pickle_loads(s, load=pickle_load):
     return load(BytesIO(s))
@@ -327,7 +336,11 @@ else:
 def register_pickle():
     """The fastest serialization method, but restricts
     you to python clients."""
-    registry.register('pickle', pickle.dumps, unpickle,
+
+    def dumps(obj, dumper=pickle.dumps):
+        return dumper(obj, protocol=pickle_protocol)
+
+    registry.register('pickle', dumps, unpickle,
                       content_type='application/x-python-serialize',
                       content_encoding='binary')
 
@@ -336,7 +349,8 @@ def register_msgpack():
     """See http://msgpack.sourceforge.net/"""
     try:
         try:
-            from msgpack import packb as dumps, unpackb as loads
+            from msgpack import packb as dumps, unpackb
+            loads = lambda s: unpackb(s, encoding='utf-8')
         except ImportError:
             # msgpack < 0.2.0 and Python 2.5
             from msgpack import packs as dumps, unpacks as loads  # noqa
