@@ -12,7 +12,7 @@ from __future__ import absolute_import
 from __future__ import with_statement
 
 from bisect import bisect
-from itertools import cycle, islice
+from contextlib import contextmanager
 from time import time
 from Queue import Empty
 
@@ -58,6 +58,33 @@ PRIORITY_STEPS = [0, 3, 6, 9]
 # queues manually.
 
 
+class MutexHeld(Exception):
+    pass
+
+
+@contextmanager
+def Mutex(client, name, expire):
+    lock_id = uuid()
+    if client.setnx(name, lock_id):
+        client.expire(name, expire)
+        yield
+    else:
+        if not client.ttl(name):
+            client.expire(name, expire)
+        raise MutexHeld()
+
+    pipe = client.pipeline(True)
+    try:
+        pipe.watch(name)
+        if pipe.get(name) == lock_id:
+            pipe.multi()
+            pipe.delete(name)
+            pipe.execute()
+        pipe.unwatch()
+    except redis.WatchError:
+        pass
+
+
 class QoS(virtual.QoS):
     restore_at_shutdown = True
 
@@ -96,20 +123,26 @@ class QoS(virtual.QoS):
         self._vrestore_count += 1
         if (self._vrestore_count - 1) % interval:
             return
+        client = self.client
         ceil = time() - self.visibility_timeout
-        visible = self.client.zrevrangebyscore(
-                self.unacked_index_key, ceil, 0,
-                start=start, num=num, withscores=True)
-        for tag, score in visible or []:
-            self.restore_by_tag(tag)
+        try:
+            with Mutex(client, self.unacked_mutex_key,
+                               self.unacked_mutex_expire):
+                visible = client.zrevrangebyscore(
+                        self.unacked_index_key, ceil, 0,
+                        start=num and start, num=num, withscores=True)
+                for tag, score in visible or []:
+                    self.restore_by_tag(tag, client)
+        except MutexHeld:
+            pass
 
-    def restore_by_tag(self, tag):
+    def restore_by_tag(self, tag, client=None):
+        client = client or self.client
         p, _, _ = self._remove_from_indices(tag,
-                        self.client.pipeline().hget(self.unacked_key, tag)) \
-                            .execute()
+            client.pipeline().hget(self.unacked_key, tag)).execute()
         if p:
             M, EX, RK = loads(p)
-            self.channel._do_restore_message(M, EX, RK)
+            self.channel._do_restore_message(M, EX, RK, client)
 
     @property
     def client(self):
@@ -123,6 +156,14 @@ class QoS(virtual.QoS):
     def unacked_index_key(self):
         return self.channel.unacked_index_key
 
+    @cached_property
+    def unacked_mutex_key(self):
+        return self.channel.unacked_mutex_key
+
+    @cached_property
+    def unacked_mutex_expire(self):
+        return self.channel.unacked_mutex_expire
+
     @cached_property
     def visibility_timeout(self):
         return self.channel.visibility_timeout
@@ -201,13 +242,17 @@ class MultiChannelPoller(object):
     def on_poll_init(self, poller):
         self.poller = poller
         for channel in self._channels:
-            channel.qos.restore_visible()
+            return channel.qos.restore_visible(
+                num=channel.unacked_restore_limit,
+            )
 
     def on_poll_empty(self):
         for channel in self._channels:
             if channel.active_queues:
                 # only need to do this once, as they are not local to channel.
-                return channel.qos.restore_visible()
+                return channel.qos.restore_visible(
+                    num=channel.unacked_restore_limit,
+                )
 
     def handle_event(self, fileno, event):
         if event & READ:
@@ -256,20 +301,26 @@ class Channel(virtual.Channel):
     _fanout_queues = {}
     unacked_key = 'unacked'
     unacked_index_key = 'unacked_index'
-    visibility_timeout = 3600  # 1 hour
+    unacked_mutex_key = 'unacked_mutex'
+    unacked_mutex_expire = 300  # 5 minutes
+    unacked_restore_limit = None
+    visibility_timeout = 3600   # 1 hour
     priority_steps = PRIORITY_STEPS
 
     from_transport_options = (virtual.Channel.from_transport_options
                             + ('unacked_key',
                                'unacked_index_key',
+                               'unacked_mutex_key',
+                               'unacked_mutex_expire',
                                'visibility_timeout',
+                               'unacked_restore_limit',
                                'priority_steps'))
 
     def __init__(self, *args, **kwargs):
         super_ = super(Channel, self)
         super_.__init__(*args, **kwargs)
 
-        self._queue_cycle = cycle([])
+        self._queue_cycle = []
         self.Client = self._get_client()
         self.ResponseError = self._get_response_error()
         self.active_fanout_queues = set()
@@ -285,27 +336,29 @@ class Channel(virtual.Channel):
         # are still waiting for data.
         self.connection_errors = self.connection.connection_errors
 
-    def _do_restore_message(self, payload, exchange, routing_key):
+    def _do_restore_message(self, payload, exchange, routing_key, client=None):
+        client = client or self._avail_client
         try:
             try:
                 payload['headers']['redelivered'] = True
             except KeyError:
                 pass
             for queue in self._lookup(exchange, routing_key):
-                self._avail_client.lpush(queue, dumps(payload))
+                client.lpush(queue, dumps(payload))
         except Exception:
             logger.critical('Could not restore message: %r', payload,
                     exc_info=True)
 
     def _restore(self, message, payload=None):
         tag = message.delivery_tag
-        P, _ = self._avail_client.pipeline() \
-                            .hget(self.unacked_key, tag) \
-                            .hdel(self.unacked_key, tag) \
-                         .execute()
+        client = self._avail_client
+        P, _ = client.pipeline() \
+                    .hget(self.unacked_key, tag) \
+                    .hdel(self.unacked_key, tag) \
+                .execute()
         if P:
             M, EX, RK = loads(P)
-            self._do_restore_message(M, EX, RK)
+            self._do_restore_message(M, EX, RK, client)
 
     def _next_delivery_tag(self):
         return uuid()
@@ -391,6 +444,7 @@ class Channel(virtual.Channel):
             if dest__item:
                 dest, item = dest__item
                 dest = dest.rsplit(self.sep, 1)[0]
+                self._rotate_cycle(dest)
                 return loads(item), dest
             else:
                 raise Empty()
@@ -585,13 +639,25 @@ class Channel(virtual.Channel):
         each queue is equally likely to be consumed from,
         so that a very busy queue will not block others.
 
+        This works by using Redis's `BRPOP` command and
+        by rotating the most recently used queue to the
+        and of the list.  See Kombu github issue #166 for
+        more discussion of this method.
+
         """
-        self._queue_cycle = cycle(self.active_queues)
+        self._queue_cycle = list(self.active_queues)
 
     def _consume_cycle(self):
         """Get a fresh list of queues from the queue cycle."""
         active = len(self.active_queues)
-        return list(islice(self._queue_cycle, 0, active + 1))[:active]
+        return self._queue_cycle[0:active]
+
+    def _rotate_cycle(self, used):
+        """
+        Move most recently used queue to end of list
+        """
+        index = self._queue_cycle.index(used)
+        self._queue_cycle.append(self._queue_cycle.pop(index))
 
     def _get_response_error(self):
         from redis import exceptions
